@@ -36,7 +36,8 @@ use constant IS_MOD_PERL_2_BUILD => IS_MOD_PERL_2 &&
 use constant IS_APACHE_TEST_BUILD =>
     grep { -e "$_/lib/Apache/TestConfig.pm" } qw(Apache-Test . ..);
 
-use Symbol ();
+use constant CUSTOM_CONFIG_FILE => 'Apache/TestConfigData.pm';
+
 use File::Copy ();
 use File::Find qw(finddepth);
 use File::Basename qw(dirname);
@@ -44,16 +45,29 @@ use File::Path ();
 use File::Spec::Functions qw(catfile abs2rel splitdir canonpath
                              catdir file_name_is_absolute devnull);
 use Cwd qw(fastcwd);
+use Socket ();
+use Symbol ();
 
 use Apache::TestConfigPerl ();
 use Apache::TestConfigParse ();
 use Apache::TestTrace;
 use Apache::TestServer ();
 use Apache::TestRun ();
-use Socket ();
 
 use vars qw(%Usage);
 
+# variables stored in $Apache::TestConfigData::vars
+my @data_vars_must = qw(httpd apxs);
+my @data_vars_opt  = qw(user group port);
+# mapping from $Apache::TestConfigData::vars to $ENV settings
+my %vars_to_env = (
+    httpd => 'APACHE_TEST_HTTPD',
+    apxs  => 'APACHE_TEST_APXS',
+    user  => 'APACHE_TEST_USER',
+    group => 'APACHE_TEST_GROUP',
+    port  => 'APACHE_TEST_PORT',
+);
+
 %Usage = (
    top_dir         => 'top-level directory (default is $PWD)',
    t_dir           => 'the t/ test directory (default is $top_dir/t)',
@@ -83,7 +97,7 @@ use vars qw(%Usage);
    sslcaorg        => 'SSL CA organization to use for tests (default is asf)',
    libmodperl      => 'path to mod_perl\'s .so (full or relative to LIBEXECDIR)',
    defines         => 'values to add as -D defines (for example, "VAR1 VAR2")',
-   (map { $_ . '_module_name', "$_ module name"} qw(cgi ssl thread access auth)),
+   (map { $_ . '_module_name', "$_ module name"} qw(cgi ssl thread access auth php)),
 );
 
 my %filepath_conf_opts = map { $_ => 1 }
@@ -173,8 +187,11 @@ sub new_test_server {
     Apache::TestServer->new($args || $self)
 }
 
+# setup httpd-independent components
+# for httpd-specific call $self->httpd_config()
 sub new {
     my $class = shift;
+
     my $args;
 
     $args = shift if $_[0] and ref $_[0];
@@ -220,7 +237,7 @@ sub new {
 
     # custom config options from Apache::TestConfigData
     # again, this should force reconfiguration
-    Apache::TestRun::custom_config_add_conf_opts($args);
+    custom_config_add_conf_opts($args);
 
     my $self = bless {
         clean => {},
@@ -282,15 +299,58 @@ sub new {
     $vars->{serveradmin}  ||= $self->default_serveradmin;
 
     $vars->{minclients}   ||= 1;
+    my $maxclientspreset = $vars->{maxclients} || 0;
     # prevent 'server reached MaxClients setting' errors
     $vars->{maxclients}   ||= $vars->{minclients} + 1;
+    # for threaded mpms MaxClients must be a multiple of
+    # ThreadsPerChild (i.e. maxclients % minclients == 0)
+    # so unless -maxclients was explicitly specified use a double of
+    # minclients
+    $vars->{maxclientsthreadedmpm} = 
+        $maxclientspreset || $vars->{minclients} * 2;
 
     $vars->{proxy}        ||= 'off';
     $vars->{proxyssl_url} ||= '';
     $vars->{defines}      ||= '';
 
+    $self->{hostport} = $self->hostport;
+    $self->{server} = $self->new_test_server;
+
+    return $self;
+
+}
+
+# figure out where httpd is and run extra config hooks which require
+# knowledge of where httpd is
+sub httpd_config {
+    my $self = shift;
+
+    my $vars = $self->{vars};
+
     $self->configure_apxs;
     $self->configure_httpd;
+
+    unless ($vars->{httpd} or $vars->{apxs}) {
+        if ($ENV{APACHE_TEST_NO_STICKY_PREFERENCES}) {
+            error "You specified APACHE_TEST_NO_STICKY_PREFERENCES=1 " .
+                "in which case you must explicitly specify -httpd " .
+                "and/or -apxs options";
+            Apache::TestRun::exit_perl(0);
+        }
+
+        $self->clean(1);
+        # this method restarts the whole program via exec
+        # so it never returns
+        $self->custom_config_first_time($self->{vars});
+    }
+
+    # if we have gotten that far we know at least about the location
+    # of httpd and or apxs, so let's save it if we haven't saved any
+    # custom configs yet
+    unless (custom_config_exists()) {
+        $self->custom_config_save($self->{vars});
+    }
+
     $self->inherit_config; #see TestConfigParse.pm
     $self->configure_httpd_eapi; #must come after inherit_config
 
@@ -299,10 +359,9 @@ sub new {
     $self->default_module(ssl    => [qw(mod_ssl)]);
     $self->default_module(access => [qw(mod_access mod_authz_host)]);
     $self->default_module(auth   => [qw(mod_auth mod_auth_basic)]);
+    $self->default_module(php    => [qw(mod_php4 mod_php5)]);
 
-    $self->{hostport} = $self->hostport;
-
-    $self->{server} = $self->new_test_server;
+    $self->{server}->post_config;
 
     $self;
 }
@@ -349,6 +408,8 @@ sub configure_httpd {
     my $self = shift;
     my $vars = $self->{vars};
 
+    debug "configuring httpd";
+
     $vars->{target} ||= (WIN32 ? 'Apache.EXE' : 'httpd');
 
     unless ($vars->{httpd}) {
@@ -699,7 +760,8 @@ sub hostport {
 sub find_apache_module {
     my($self, $module) = @_;
 
-    die "find_apache_module: module name argument is required" unless $module;
+    die "find_apache_module: module name argument is required"
+        unless $module;
 
     my $vars = $self->{vars};
     my $sroot = $vars->{serverroot};
@@ -874,6 +936,11 @@ sub perlscript_header {
         }
     }
 
+    {
+        my $dir = canonpath catdir $FindBin::Bin, "t", "lib";
+        push @dirs, $dir if -d $dir;
+    }
+
     my $dirs = join("\n    ", '', @dirs) . "\n";;
 
     return <<"EOF";
@@ -1302,6 +1369,11 @@ sub sslca_generate {
 sub sslca_clean {
     my $self = shift;
 
+    # XXX: httpd config is required, for now just skip ssl clean if
+    # there is none. should probably add some flag which will tell us
+    # when httpd_config was already run
+    return unless $self->{vars}->{httpd} && $self->{vars}->{ssl_module};
+
     return unless $self->sslca_can;
 
     Apache::TestSSLCA::clean($self);
@@ -1536,6 +1608,7 @@ sub which {
 sub apxs {
     my($self, $q, $ok_fail) = @_;
     return unless $self->{APXS};
+    local @ENV{ qw(PATH IFS CDPATH ENV BASH_ENV) };
     my $devnull = devnull();
     my $apxs = shell_ready($self->{APXS});
     my $val = qx($apxs -q $q 2>$devnull);
@@ -1681,10 +1754,461 @@ sub as_string {
 # escape quotes)
 sub shell_ready {
     my $arg = shift;
-    $arg =~ s/"/\"/g;
+    $arg =~ s!\\?"!\\"!g;
     return qq["$arg"];
 }
 
+### Permanent custom configuration functions ###
+
+# determine which configuration file Apache/TestConfigData.pm to use
+# (as there could be several). The order searched is:
+# 1) $ENV{HOME}/.apache-test/
+# 2) in @INC
+my $custom_config_path;
+sub custom_config_path {
+
+    return $custom_config_path if $custom_config_path;
+
+    my @inc  = ();
+
+    # XXX $ENV{HOME} isn't propagated in mod_perl
+    push @inc, catdir $ENV{HOME}, '.apache-test' if $ENV{HOME};
+
+    push @inc, @INC;
+
+    for (@inc) {
+        my $candidate = File::Spec->rel2abs(catfile $_, CUSTOM_CONFIG_FILE);
+        next unless -e $candidate;
+        return $custom_config_path = $candidate;
+    }
+
+    return '';
+}
+
+sub custom_config_exists {
+    # try to load custom config if it wasn't loaded yet (there are
+    # many entry points to this API)
+    custom_config_load();
+
+    # it's enough to check whether we have a custom
+    # config for 'httpd' or 'apxs'.
+    my $httpd = $Apache::TestConfigData::vars->{httpd} || '';
+    return 1 if $httpd && -e $httpd && -x _;
+
+    my $apxs = $Apache::TestConfigData::vars->{apxs} || '';
+    return 1 if $apxs && -e $apxs && -x _;
+
+    return 0;
+}
+
+# to be used only from Apache-Test/Makefile.PL to write the custom
+# configuration module so it'll be copied to blib during 'make' and
+# updated to use custom config data during 'make test' and then
+# installed system-wide via 'make install'
+#
+# it gets written only if the custom configuration didn't exist
+# already
+sub custom_config_file_stub_write {
+
+    return if custom_config_exists();
+
+    # It doesn't matter whether it gets written under modperl-2.0/lib
+    # or Apache-Test/lib root, since Apache::TestRun uses the same
+    # logic and will update that file with real config data, which
+    # 'make install' will then pick and install system-wide. but
+    # remember that $FindBin::Bin is the location of top-level
+    # 'Makefile.PL'
+    require FindBin; # load it only for this particular use
+    my $path = catfile $FindBin::Bin, "lib",
+        Apache::TestConfig::CUSTOM_CONFIG_FILE;
+
+    # write an empty stub
+    Apache::TestConfig::custom_config_write($path, '');
+}
+
+sub custom_config_save {
+    my $self = shift;
+    my $conf_opts = shift;
+
+    if ($ENV{APACHE_TEST_NO_STICKY_PREFERENCES}) {
+        debug "APACHE_TEST_NO_STICKY_PREFERENCES=1 => " .
+            "skipping save of custom config data";
+        return;
+    }
+
+    my $vars = $self->{vars};
+    my $config_dump = '';
+
+    # minimum httpd and/or apxs needs to be set
+    return 0 unless $vars->{httpd} or $Apache::TestConfigData::vars->{httpd}
+        or          $vars->{apxs}  or $Apache::TestConfigData::vars->{apxs};
+
+    # it doesn't matter how these vars were set (httpd may or may not
+    # get set using the path to apxs, w/o an explicit -httpd value)
+    for (@data_vars_must) {
+        next unless my $var = $vars->{$_} || $conf_opts->{$_};
+        $config_dump .= qq{    '$_' => '$var',\n};
+    }
+
+    # save these vars only if they were explicitly set via command line
+    # options. For example if someone builds A-T as user 'foo', then
+    # installs it as root and we save it, all users will now try to
+    # configure under that user 'foo' which won't quite work.
+    for (@data_vars_opt) {
+        next unless my $var = $conf_opts->{$_};
+        $config_dump .= qq{    '$_' => '$var',\n};
+    }
+
+    if (IS_APACHE_TEST_BUILD) {
+        my $path = catfile $vars->{top_dir}, 'lib', CUSTOM_CONFIG_FILE;
+        # if it doesn't exist, then we already have a global config file
+        # if it does, then we have need to update it and its blib/ copy
+        if (-e $path and custom_config_path_is_writable($path)) {
+            custom_config_write($path, $config_dump);
+            # also update blib/lib, since usually that's the one that
+            # appears in @INC when t/TEST is run. and it won't be
+            # synced with blib/ unless 'make' was run
+            my $blib_path = catfile $vars->{top_dir},
+                'blib', 'lib', CUSTOM_CONFIG_FILE;
+            if (-e $blib_path and custom_config_path_is_writable($blib_path)) {
+                custom_config_write($blib_path, $config_dump);
+            }
+            return 1;
+        }
+    }
+
+    my $path;
+    if ($path = custom_config_path() ) {
+        # do nothing, the config file already exists (global)
+        debug "Found custom config '$path'";
+    }
+    elsif (File::Spec->file_name_is_absolute(__FILE__)) {
+        # next try a global location, as if it was configured before
+        # Apache::Test's 'make install' (install in the same dir as
+        # Apache/TestRun.pm)
+        # if the filename is not absolute that means that we are still
+        # in Apache-Test build (could just test for IS_APACHE_TEST_BUILD)
+        my $base = dirname dirname __FILE__;
+        $path = catdir $base, CUSTOM_CONFIG_FILE;
+    }
+
+    # check whether we can write to the directory of the chosen path
+    # (e.g. root-owned directory)
+    if ($path and custom_config_path_is_writable($path)) {
+        custom_config_write($path, $config_dump);
+        return 1;
+    }
+    # if we have no writable path yet, try to use ~
+    elsif ($ENV{HOME}) {
+        $path = catfile $ENV{HOME}, '.apache-test', CUSTOM_CONFIG_FILE;
+        if ($path and custom_config_path_is_writable($path)) {
+            custom_config_write($path, $config_dump);
+            return 1;
+        }
+    }
+
+    # XXX: should we croak since we failed to write config
+    error "Failed to find a config file to save the custom " .
+        "configuration in";
+    return 0;
+}
+
+sub custom_config_path_is_writable {
+    my $path = shift;
+
+    return 0 unless $path;
+
+    my $file_created    = '';
+    my $top_dir_created = '';
+    # first make sure that the file is writable if it exists
+    # already (it might be non-writable if installed via EU::MM or in
+    # blib/)
+    if (-e $path) {
+        my $mode = (stat _)[2];
+        $mode |= 0200;
+        chmod $mode, $path; # it's ok if we fail
+        # keep it writable if we have changed it from not being one
+        # so that custom_config_save will be able to just overwrite it
+    }
+    else {
+        my $dir = dirname $path;
+        if ($dir and !-e $dir) {
+            my @dirs = File::Path::mkpath($dir, 0, 0755);
+            # the top level dir to nuke on cleanup if it was created
+            $top_dir_created = shift @dirs if @dirs;
+        }
+        # not really create yet, but will be in the moment
+        $file_created = 1;
+    }
+
+    # try to open for append (even though it may not exist
+    my $fh = Symbol::gensym;
+    if (open $fh, ">>$path") {
+        close $fh;
+        # cleanup if we just created the file
+        unlink $path if $file_created;
+        File::Path::rmtree([$top_dir_created], 0, 0) if $top_dir_created;
+        return 1;
+    }
+
+    return 0;
+}
+
+sub custom_config_write {
+    my($path, $config_dump) = @_;
+
+    my $pkg = << "EOC";
+package Apache::TestConfigData;
+
+use strict;
+use warnings;
+
+\$Apache::TestConfigData::vars = {
+$config_dump
+};
+
+1;
+
+=head1 NAME
+
+Apache::TestConfigData - Configuration file for Apache::Test
+
+=cut
+EOC
+
+    debug "Writing custom config $path";
+    my $dir = dirname $path;
+    File::Path::mkpath($dir, 0, 0755) unless -e $dir;
+    my $fh = Symbol::gensym;
+    open $fh, ">$path" or die "Cannot open $path: $!";
+    print $fh $pkg;
+    close $fh;
+}
+
+sub custom_config_add_conf_opts {
+    my $args = shift;
+
+    return unless $Apache::TestConfigData::vars and 
+        keys %$Apache::TestConfigData::vars;
+
+    debug "overlaying custom config data";
+
+    # the logic is quite complicated with 'httpd' and 'apxs', since
+    # one is enough to run the test suite, and we need to avoid the
+    # situation where both are saved in custom config but only one
+    # (let's say httpd) is overriden by the command line /env var and
+    # a hell may break loose if we take that overriden httpd value and
+    # also use apxs from custom config which could point to a different
+    # server. So if there is an override of apxs or httpd, do not use
+    # the custom config for apxs or httpd.
+    my $vars_must_overriden = grep {
+        $ENV{ $vars_to_env{$_} } || $args->{$_}
+    } @data_vars_must;
+
+    # mod_perl 2.0 build always knows the right httpd location (and
+    # optionally apxs)
+    $vars_must_overriden++ if IS_MOD_PERL_2_BUILD();
+
+    unless ($vars_must_overriden) {
+        for (@data_vars_must) {
+            next unless $Apache::TestConfigData::vars->{$_};
+            $args->{$_} = $Apache::TestConfigData::vars->{$_};
+        }
+    }
+
+    for (@data_vars_opt) {
+        next unless $Apache::TestConfigData::vars->{$_};
+        # env vars override custom config
+        my $env_value = $ENV{ $vars_to_env{$_} };
+        next unless defined $env_value and length $env_value;
+        $args->{$_} ||= $Apache::TestConfigData::vars->{$_};
+    }
+}
+
+my $custom_config_loaded = 0;
+sub custom_config_load {
+
+    if ($ENV{APACHE_TEST_NO_STICKY_PREFERENCES}) {
+        debug "APACHE_TEST_NO_STICKY_PREFERENCES=1 => " .
+            "skipping load of custom config data";
+        return;
+    }
+
+    return if $custom_config_loaded;
+
+    if (my $custom_config_path = custom_config_path()) {
+        debug "loading custom config data from: '$custom_config_path'";
+        $custom_config_loaded++;
+        require $custom_config_path;
+    }
+    else {
+        debug "no custom config data was loaded";
+    }
+}
+
+sub custom_config_first_time {
+    my $self = shift;
+    my $conf_opts = shift;
+
+    my $vars = $self->{vars};
+
+    print qq[
+
+We are now going to configure the Apache-Test framework.
+This configuration process needs to be done only once.
+
+];
+
+    print qq[
+
+First we need to know where the 'httpd' executable is located.
+If you have more than one Apache server is installed, make sure
+you supply the path to the one you are going to use for testing.
+You can always override this setting at run time via the '-httpd'
+option. For example:
+
+  % t/TEST -httpd /path/to/alternative/httpd
+
+or via the environment variable APACHE_TEST_HTTPD. For example:
+
+  % APACHE_TEST_HTTPD=/path/to/alternative/httpd t/TEST
+
+If for some reason you want to skip the test suite, type: skip
+];
+
+    {
+        my %choices = ();
+        my @tries = qw(httpd httpd2);
+        # Win32 uses Apache or perhaps Apache2, not apache/apache2
+        push @tries, WIN32 ? qw(Apache Apache2) : qw(apache apache2);
+        for (grep defined $_,
+             map({ catfile $vars->{$_}, $vars->{target} } qw(sbindir bindir)),
+             $self->default_httpd, which($vars->{target}),
+             $ENV{APACHE}, $ENV{APACHE2},
+             $ENV{APACHE_TEST_HTTPD}, $ENV{APACHE_TEST_HTTPD2},
+             map {which($_)} @tries) {
+            $choices{$_}++ if -e $_ && -x _;
+        }
+        my $optional = 0;
+        my $wanted = 'httpd';
+        $vars->{$wanted} = 
+            _custom_config_prompt_path($wanted, \%choices, $optional);
+    }
+
+    print qq[
+
+Next we need to know where the 'apxs' script is located. This script
+provides a lot of information about the apache installation, and makes
+it easier to find things. However it's not available on all platforms,
+therefore it's optional.
+
+If you don't have it installed it's not a problem. Just press Enter.
+
+Notice that if you have Apache 2.x installed that script could be
+called as 'apxs2'.
+
+If you have more than one Apache server is installed, make sure you
+supply the path to the apxs script you are going to use for testing.
+You can always override this setting at run time via the '-apxs'
+option. For example:
+
+  % t/TEST -apxs /path/to/alternative/apxs
+
+or via the environment variable APACHE_TEST_APXS. For example:
+
+  % APACHE_TEST_APXS=/path/to/alternative/apxs t/TEST
+
+];
+    {
+        my %choices = ();
+        for (grep defined $_,
+             map({ catfile $vars->{$_}, 'apxs' } qw(sbindir bindir)),
+             $self->default_apxs,
+             $ENV{APXS},  $ENV{APACHE_TEST_APXS},  which('apxs'),
+             $ENV{APXS2}, $ENV{APACHE_TEST_APXS2}, which('apxs2')) {
+            $choices{$_}++ if -e $_ && -x _;
+        }
+        my $optional = 1;
+        my $wanted = 'apxs';
+        $vars->{$wanted} = 
+            _custom_config_prompt_path($wanted, \%choices, $optional);
+    }
+
+    $self->custom_config_save($conf_opts);
+
+    # we probably could reconfigure on the fly ($self->configure), but
+    # the problem is various cached data which won't be refreshed. so
+    # the simplest is just to restart the run from scratch
+    Apache::TestRun::rerun();
+}
+
+sub _custom_config_prompt_path {
+    my($wanted, $rh_choices, $optional) = @_;
+
+    my $ans;
+    my $default = '';
+    my $optional_str = $optional ? " (optional)" : '';
+    my $prompt =
+        "\nPlease provide a full path to$optional_str '$wanted' executable";
+
+    my @choices = ();
+    if (%$rh_choices) {
+        $prompt .= " or choose from the following options:\n\n";
+        my $c = 0;
+        for (sort keys %$rh_choices) {
+            $c++;
+            $prompt .= "    [$c] $_\n";
+            push @choices, $_;
+        }
+        $prompt .= " \n";
+        $default = 1; # a wild guess
+    }
+    else {
+        $prompt .= ":\n\n";
+    }
+
+    while (1) {
+        $ans = ExtUtils::MakeMaker::prompt($prompt, $default);
+
+        # strip leading/closing spaces
+        $ans =~ s/^\s*|\s*$//g;
+
+        # convert the item number to the path
+        if ($ans =~ /^(\d+)$/) {
+            if ($1 > 0 and $choices[$1-1]) {
+                $ans = $choices[$1-1];
+            }
+            else {
+                warn "The choice '$ans' doesn't exist\n";
+                next;
+            }
+        }
+
+        if ($optional) {
+            return '' unless $ans;
+        }
+
+        # stop the test suite without an error (so automatic tools
+        # like CPAN.pm will be able to continue)
+        if (lc($ans) eq 'skip' && !$optional) {
+            skip_test_suite();
+            next; # in case they change their mind
+        }
+
+        unless (File::Spec->file_name_is_absolute($ans)) {
+            my $cwd = Cwd::cwd();
+            warn "The path '$ans' is not an absolute path. " .
+                "Please specify an absolute path\n";
+            next;
+        }
+
+        warn("'$ans' doesn't exist\n"),     next unless -e $ans;
+        warn("'$ans' is not executable\n"), next unless -x $ans;
+
+        return $ans;
+    }
+}
+
 1;
 
 =head1 NAME
@@ -1863,7 +2387,7 @@ HostnameLookups Off
     MinSpareThreads      @MinClients@
     MaxSpareThreads      @MinClients@
     ThreadsPerChild      @MinClients@
-    MaxClients           @MaxClients@
+    MaxClients           @MaxClientsThreadedMPM@
     MaxRequestsPerChild  0
 </IfModule>
 
@@ -1893,7 +2417,7 @@ HostnameLookups Off
 </IfDefine>
 
 <IfModule mpm_winnt.c>
-    ThreadsPerChild      25
+    ThreadsPerChild      50
     MaxRequestsPerChild  0
 </IfModule>
 
