@@ -63,8 +63,30 @@ sub skip {
 sub run_t {
     my($self, $file) = @_;
     my $ran = 0;
-    my $lib = catfile Apache::Test::vars('top_dir'), qw(Apache-Test lib);
-    my $cmd = qq[$^X -Mlib="$lib" $file];
+
+    my $source_lib = '';
+
+    if (Apache::TestConfig::IS_APACHE_TEST_BUILD) {
+        # so we can find Apache/Test.pm from both the perl-framework/
+        # and Apache-Test/
+
+        my $top_dir = Apache::Test::vars('top_dir');
+
+        foreach my $lib (catfile($top_dir, qw(Apache-Test lib)),
+                         catfile($top_dir, 'lib')) {
+
+            if (-d $lib) {
+
+                info "adding source lib $lib to \@INC";
+
+                $source_lib = qq[-Mlib="$lib"];
+
+                last;
+            }
+        }
+    }
+    
+    my $cmd = qq[$^X $source_lib $file];
 
     my $h = Symbol::gensym();
     open $h, "$cmd|" or die "open $cmd: $!";
